Documentation - Jobs
====================

Every job must implement the interface `SlmQueue\Job\JobInterface`. For ease of use, an abstract class 
`SlmQueue\Job\AbstractJob` is provided to implement most of the interface methods. The method, `execute()`, must be
implemented in userland code.

An example job would look like the following:

```php
namespace MyModule\Job;

use SlmQueue\Job\AbstractJob;

class PrintHelloWorldJob extends AbstractJob
{
    public function execute()
    {
        echo 'Hello World';
    }
}
```

Job payload
-----------
The job often needs some data to work with, commonly called a payload. For an email job, the contents of the email (to
address, subject and message) are the payload. If you generate an invoice based on an order in your database, you set
the id of this order as the payload. 

Internally, a `SlmQueue\Job\AbstractJob` extends the `Zend\Stdlib\Message` class which allows it to set and get content.
These methods are used to set the payload upon pushing and retrieve the payload when the job is executed.

To set the payload in a controller, below is an example:

```php
namespace MyModule\Controller;

use Zend\Mvc\Controller\AbstractActionController;

class MyController extends AbstractActionController
{
    public function fooAction()
    {
        // Do some work

        $job = $this->jobManager->get('MyModule\Job\SendEmailJob');
        $job->setContent(array(
            'to'      => 'bob@acme.com',
            'subject' => 'Registration completed',
            'message' => 'Hi bob, you just registered for our website! Welcome!'
         ));

        $this->queue->push($job);
    }
}
```

Then you can fetch this payload from the job itself:

```php
namespace MyModule\Job;

use SlmQueue\Job\AbstractJob;

class SendEmailJob extends AbstractJob
{
    public function execute()
    {
        $payload = $this->getContent();
        $to      = $payload['to'];
        $subject = $payload['subject'];
        $message = $payload['message'];

        // Do something with $to, $subjet, $message
    }
}
```

Job payload are a flexible array structure and will be automatically serialized by SlmQueue. This means you can have
values like a `DateTime` object as payload for jobs which will be serialized and deserialized in the background. However,
this also give you restrictions than payload data *must* be serializable. Doctrine 2 entities, references to 
`Zend\Mvc\Application` or other unserializable instances should not be set as payload.

Job dependencies
----------------

Because of the configuration via a job plugin manager, you can inject dependencies you need into the constructor of the
job class. This will require you to define a job factory for that job as well. Assume here we send an email in a job and
this job requires the email transport class as a dependency.

```php
namespace MyModule\Job;

use SlmQueue\Job\AbstractJob;
use Zend\Mail\Transport\TransportInterface;
use Zend\Mail\Message;

class SendEmailJob extends AbstractJob
{
    protected $transport;

    public function __construct(TransportInterface $transport)
    {
        $this->transport = $transport;
    }

    public function execute()
    {
        $message = new Message;
        $payload = $this->getContent();

        $message->setTo($payload['to']);
        $message->setSubject($payload['subject']);
        $message->setContent($payload['content']);

        $this->transport->send($message);
    }
}
```

To inject the email transport instance, a factory must be created to instantiate the job:

```php
namespace MyModule\Factory;

use Zend\ServiceManager\FactoryInterface;
use Zend\ServiceManager\ServiceLocatorInterface;

use MyModule\Job\SendEmailJob;

class SendEmailJobFactory implements FactoryInterface
{
    public function createService(ServiceLocatorInterface $sl)
    {
        $transport = $sl->getServiceLocator()->get('my-transport-service');

        $job = new SendEmailJob($transport);
        return $job;
    }
}
```

The last step is to register this factory for the above job:

```php
'slm_queue' => array(
    'factories' => array(
        'MyModule\Job\PrintHelloWorldJob' => 'MyModule\Factory\SendEmailJobFactory',
    ),
)
```

Navigation
----------

Previous page: [Configuration](2.Configuration.md)
Next page: [Queue Aware](4.QueueAware.md)

1. [Introduction](1.Introduction.md)
2. [Configuration](2.Configuration.md)
3. [Jobs](3.Jobs.md)
4. [QueueAware](4.QueueAware.md)
5. [Workers](5.Workers.md)
6. [Events](6.Events.md)
7. [Worker management](7.WorkerManagement.md)
